<?
/**
 * Classe per gestire i file di configurazione.
 *
 * Un file di configurazione, per questa classe,
 * e` un file di righe nel formato
 *
 *   <chiave><separatore><valore>
 *
 * dove <separatore> e` una stringa, es. ":" o " = "
 *
 * N.B. se separator e` vuoto, il file gestisce una
 *      lista senza duplicati.
 *      Questo puo` servire per esempio per
 *      /etc/sendmail.cw
 *
 * Scritta da Lorenzo Colitti <colitti@xmedia.net>
 * Copyright 2000-2001 xmedia srl - http://www.xmedia.net/
 **/

	class configfile {
		// Il nome del file di configurazione
		var $filename;

		// L'array che lo rappresenta
		var $arr;

		// Il separatore tra chiave e valore
		var $separator;

		function configfile($filename, $separator = ":") {
			$this->filename = $filename;
			$this->comparefunction = $comparefunction;
			$this->separator = $separator;

			$this->arr = array();
		}

		// Funzione per controllare se una riga e` relativa ad una certa chiave
		function keymatches($line, $key) {
			$line = trim($line);

			$keylen = @strpos($line, $this->separator);

			// Se non c'e' separatore, considero tutta la riga come chiave
			if($keylen === false)
				$keylen = strlen($line);

			// Se il separatore e' nella prima colonna, non matcha
			// a meno che la chiave non sia vuota
			if($keylen == 0)
				if($key == "")
					return true;
				else
					return false;

			if(strncmp($line, $key, $keylen) == 0)
				return true;
			else
				return false;
		}

		// Funzione per trovare il numero di riga di una certa chiave
		function findline($key) {
			$len = strlen($key);

			reset($this->arr);
			while(list($num,$line) = each($this->arr))
				if(! ($this->keymatches($line, $key) === false) )
					return $num;

			return false;
		}

		// Funzione per cancellare una chiave
		function delete($key) {
			$count = count($this->arr);
			$linenum = $this->findline($key);

			if($linenum === false)
				return false;

			if($linenum < $count - 1)
				$this->arr[$linenum] = array_pop($this->arr);
			else
				array_pop($this->arr);

			return true;
		}

		// Funzione per trovare il valore di una certa chiave
		function get($key) {
			$linenum = $this->findline($key);

			if($linenum === false)
				return false;

			$line = $this->arr[$linenum];
			$val = trim(strstr($line, $this->separator));
			$val = substr($val, 1);

			return $val;
		}

		// Funzione per settare il valore di una chiave.
		// Se il la chiave non esiste viene creata.
		function set($key, $val) {
			$linenum = $this->findline($key);

			if($linenum === false)
				$this->arr[] = $key . $this->separator . $val . "\n";
			else
				$this->arr[$linenum] = $key . $this->separator . $val . "\n";

			return true;
		}

		// Funzione per leggere il file
		function read() {
			$this->arr = @file($this->filename);

			if(is_array($this->arr))
				return true;
			else {
				$this->arr = array();
				return false;
			}
		}

		// Funzione per scrivere il file
		function write() {
			$fp = @fopen($this->filename, "w");
			if(! $fp)
				return false;

			reset($this->arr);
			while(list($num, $line) = each($this->arr))
				if(fputs($fp, $line) == -1)
					return false;

			fclose($fp);
			return true;
		}

		// Funzione per resettare il puntatore
		function rewind() {
			reset($this->arr);
		}

		function next() {            
			$line = current($this->arr);

			next($this->arr);
 
			if($line === false)
				return false;

			$line = trim($line);
			$ret = explode($this->separator, $line, 2);

//			$len = strpos($line, $this->separator);
//			$key = substr($line, 0, $len);
//
//			$val = trim(strstr($line, $this->separator));
//			$val = substr($val, 1);
//
//			return array("key" => $key, "val" => $val);

			return $ret;
		}
	}
?>
