<?
	// This file is copyright 2000-2001 xmedia srl
	// http://www.xmedia.net/

	include("defines.php");
	include("quotadata.php");
	include("domain.php");
	include("configfile.php");

	function comparedomain($a, $b) {
		return strcmp($a->getname(), $b->getname());
	}

	function isvaliddomainname($str) {
		// Definisco un dominio come 1 o piu` [a-z0-9\-]+\.
		// seguiti da un [a-z]{1,3}
		// No, definisco un dominio come [a-z0-9][a-z0-9\-]*
		// seguito da un [a-z]{1,3}
		// perche' non puo' iniziare con il carattere -
 
		if(preg_match("/^([a-z0-9][a-z0-9\-]*\.){1,}[a-z]{1,3}$/", $str))
			return true;
	}

	function getmaindomain($user) {
		$userarray = @posix_getpwnam($user);

		if(!$userarray)
			return false;

		$comment = $userarray["gecos"];

		$len = strlen(COMMENTSTR);
		if(strncmp($comment, COMMENTSTR, $len))
			return false;

		return substr($comment, $len);
	}

	function restartqmail() {
		$cmd = PROGRAMS_DIR . "/restartqmail";
		system($cmd);
	}

	class Domains {
		var $domains;

		function writefile() {
			usort($this->domains, "comparedomain");

			@$f = fopen(DOMAINS_FILE, "w");
			if(! $f)
				return false;

			reset($this->domains);

			while(list($i, $domain) = each($this->domains))
				@fputs($f, $domain->name . ":" . $domain->user. "\n");

			@fclose($f);
		}

		function readfile() {
			$domainsfile = new configfile(DOMAINS_FILE);
			if(! $domainsfile->read())
				return false;

			$domainsfile->rewind();

			while (list($name, $user) = $domainsfile->next()) {
				$maindomain = getmaindomain($user);

				if($name == $maindomain)
					$this->domains[] = new Domain($name, $user);
				else
					$this->domains[] = new Domain($name, $user, $maindomain);
			}

			return true;
		}

		function get($name) {
			reset($this->domains);
			while(list($i, $domain) = each($this->domains))
				if($domain->getname() == $name)
					return $domain;

			return false;
		}

		function exists($name) {
			reset($this->domains);
			while(list($i, $domain) = each($this->domains))
				if($name == $domain->name)
					return true;

			return false;
		}

		function add($name, $user, $password, $aliasOf = "") {
			$name = strtolower($name);
			$user = strtolower($user);

			// Is this going to be the first domain in the array?
			if(! is_array($this->domains))
				if($aliasOf == "")
					$this->domains = array();
				else
					// Can't create an alias if there are no domains!
					return DOMAIN_NONEXISTENT;

			// Input checking
			if(! isvaliddomainname($name))
				return DOMAIN_INVALIDNAME;

			if(strlen($name) > 63)
				return DOMAIN_TOOLONG;

			if($this->exists($name)) {
				return DOMAIN_EXISTS;
			}

			// If the user already exists, make this domain an alias
			// of the user's main domain
			if($maindomain = getmaindomain($user))
				$aliasOf = getmaindomain($user);

			if($aliasOf != "") {
				$maindomain = $this->get($aliasOf);

				if($maindomain === false)
					return DOMAIN_NONEXISTENT;

				if($maindomain->isAlias())
					return DOMAIN_ALIASALIAS;

				$user = $maindomain->getuser();
			}

			// Is user name valid?
			if(!preg_match("/^[a-z][a-z\-]{0,28}[a-z]$/", $user))
				return DOMAIN_INVALIDUSER;

			// Create domain
			$domain = new Domain($name, $user, $aliasOf);
			if($aliasOf == "")
				$domain->setquotadefaults();
			else
				$domain->setaliasquotavalues();

			$domain->writequotadata();
			array_push($this->domains, $domain);

			// If it's not an alias, create user, maildir, etc.

			if($aliasOf == "") {
				$params = EscapeShellArg($user) . " " . EscapeShellArg($name);
				$params .= " " . EscapeShellArg(crypt($password));

				$cmd = PROGRAMS_DIR . "/adduser " . $params;
				system($cmd);

				$cmd = PROGRAMS_DIR . "/maildirmake " . $params;
				system($cmd);

				$cmd = PROGRAMS_DIR . "/domainsetup " . $params;
				system($cmd);
			}

			// Update rcpthosts

			$cf = new configfile("/var/qmail/control/rcpthosts", "");
			$cf->read();
			$cf->set($name, "");
			if($cf->write())
				return SUCCESS;

			// Restart qmail -- NO!!! Prima devo scrivere virtualdomains

			return FILE_ERROR;
		}

		function delete_entry($num) {
				// If it's not the last entry in the list, replace
				// it with the last entry and shorten the array.
				// If not, just shorten the array. (array is zero-based)

				$name = $this->domains[$num]->getname();
				$count = count($this->domains);

				if($num < $count - 1)
					$this->domains[$num] = array_pop($this->domains);
				else
					array_pop($this->domains);

				// Update rcpthosts
				$cf = new configfile("/var/qmail/control/rcpthosts", "");
				$cf->read();
				$cf->delete($name);
				$cf->write();
		}

		function delete($todelete) {
			$domain = $this->get($todelete);
			if($domain === false)
				return false;

			$user = $domain->getuser();
			$name = $domain->getname();

			if($domain->isalias())
				$isalias = true;
			else
				$isalias = false;

			// Step through the array.
			// If the domain was an alias, just delete it.
			// If not, it was the user's main domain: delete all its
			// aliases as well.

			reset($this->domains);

			if($isalias)
				while(list($i, $current) = each($this->domains)) {
					if($current->getname() == $todelete) {
						$current->deletequotadata(QUOTA_FILE);
						$this->delete_entry($i);	// Updates rcpthosts as well
					}
				}
			else
				while(list($i, $current) = each($this->domains)) {
					if($current->getuser() == $user) {
						$current->deletequotadata(QUOTA_FILE);
						$this->delete_entry($i);	// Updates rcpthosts as well
						$delete_user = true;
					}
				}

			if($delete_user) {
				$params = EscapeShellCmd($user);
				$params = $params . " " . EscapeShellCmd($name);

				$cmd = PROGRAMS_DIR . "/deluser " . $params;
				system($cmd);
			}

			return true;
		}

		function rewind() {
			reset($this->domains);
		}

		function next() {
			$current = current($this->domains);

			next($this->domains);

			if($current)
				return $current;
			else
				return false;
		}
	}
?>
